/*
 * This File is part of the iowj-project   
 * $Id: LCD.java,v 1.1.1.1 2006/01/15 16:47:27 Thomas Wagner Exp $
 * Copyright (C)2004 by Thomas Wagner
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.wagner_ibw.iow.lcd;

/**
 * This is the interface for the LCD special mode function.
 * It supports only single controler LCD display.
 * @author Thomas Wagner
 */
public interface LCD {

	public int getRows();
	
	public int getCols();
		
	/** OK
	 * Clears entire dislplay and sets DDRAM address 0 in address counter.
	 * @return Number of written bytes (8 expected for a successful operation).
	 */
	public long clearLCD();
	
	/** OK
	 * Sets DDRAM address 0 in address counter. Also returns display from being
	 * shiftet to original position. DDRAM contents remain unchanged.
	 * @return Number of written bytes (8 expected for a successful operation).
	 */
	public long setCursorHome();
	
	/** OK
	 * Sets cursor move direction and specifies display shift.
	 * These operations are performed during data write and read.
	 * @param moveForward true: increment, false: decrement
	 * @param shiftDisp true: display is shifted, false: display is not shifted
	 * @return Number of written bytes (8 expected for a successful operation).
	 */
	public long setEntryMode(boolean moveForward, boolean shiftDisp);
	
	/** OK
	 * Sets ON/OFF of all display (dispOn), cursor ON/OFF (cursorOn), and blink
	 * of cursor position character (cursorBlink).
	 * @param dispOn true: the display is on, false: display is off
	 * @param cursorOn true: cursor is displayed, false: cursor is not displayed
	 * @param charBlinking true: the character indicated by the cursor blinks,
	 * false: blinks not
	 * @return Number of written bytes (8 expected for a successful operation).
	 */	
	public long setDisplayControl(boolean dispOn, boolean cursorOn, boolean charBlinking);
	
	/** OK
	 * Moves cursor and shifts display whitout changing DDRAM contents.
	 * @param shiftDisp true: display shift, false: cursor move;
	 * @param shiftDir true: shift to the right, false: shift to the left
	 * @return Number of written bytes (8 expected for a successful operation).
	 */
	public long setShiftControl(boolean shiftDisp, boolean shiftDir);

	/**
	 * Write the given String in row specified by parm row.
	 * If parm clear is true the row will be cleared before output.
	 * All characters exceeding the cols will be truncated.
	 * @param row
	 * @param clear
	 * @param str
	 */
	public void writeLine(int row, boolean clear, String str)
		throws IllegalArgumentException;
		
	
	/**
	 * Write the given String in the specified row and column.
	 * If parm clear is true the row will be cleared before output.
	 * All characters exceeding the cols will be truncated.
	 * @param row
	 * @param clear
	 * @param str
	 */
	public void writeLine(int row, int col, boolean clear, String str)
		throws IllegalArgumentException;	
	
	/** OK
	 * Write the given string to LCD.
	 * @param str String to write
	 */ 
	public void writeString(String str);
	
	/** ?????
	 * Set DDRAM address
	 * Parameters:
	 * Address - DDRAM address
	 */ 
	public long setDDRAMAddr(int address);
	
	/** ?????
	 * Set DDRAM address
	 * Parameters:
	 * Address - DDRAM address
	 */ 
	public long setCGRAMAddr(int address);
	
	/** OK
	 * Moves cursor to new Position.
	 * @param row display line (1...4)
	 * @param col column in row (1...40)
	 */
	public long setCursor(int row, int col) throws IllegalArgumentException;	
	/** OK
	 * Convenient method: Turn LCD display on.
	 * @return Number of written bytes (8 expected for a successful operation).
	 */
	public long setCursorDispOn();
	
	/** OK
	 * Convenient method: Turn LCD display off.
	 * @return Number of written bytes (8 expected for a successful operation).
	 */ 
	public long setDispOff();
	
	/** OK
	 * Convenient method: Turn LCD cursor on.
	 * @return Number of written bytes (8 expected for a successful operation).
	 */ 
	public long setCursorOn();
	
	/** OK
	 * Convenient method: Turn LCD cursor off.
	 * @return Number of written bytes (8 expected for a successful operation).
	 */ 
	public long setCursorOff();
	
	/**
 	 * Move LCD cursor to the left
 	 * @return Number of written bytes (8 expected for a successful operation).
 	 */
	public long setCursorleft();
	
	/**
	 * Move LCD cursor to the right
	 * @return Number of written bytes (8 expected for a successful operation).
	 */
	public long setCursorRight();

	/**
	 * Prft die korrekte Anzeige.
	 *
	 */
	public void check();
		
	public void setSpecialChar(int code, int[]pattern) throws IllegalArgumentException;
	
	public void moveSprite(int row, String[] sprites, int wait) throws IllegalArgumentException;
	
	}
